require 'benchmark'

module Benchmark
  class ColoredReport
    def initialize(width = 0, format = nil)
      @width, @format, @list = width, format, []
    end

    def item(label = "", min_value = 0.0, max_value = 1.1, *format, &blk) # :yield:
      @list << res = Benchmark.measure(label, &blk)
      real_value = res.real

      if real_value > min_value and real_value < max_value
        print label.to_s.ljust(@width).green
        print res.format(@format, *format).green
      else
        print label.to_s.ljust(@width).red
        print res.format(@format, *format).red
      end
      res
    end

    alias report item

    attr_reader :list
  end

  def self.penchmark(caption = '', label_width = nil, format = nil, *labels)

    label_width ||= 0
    label_width += 1
    format ||= FORMAT
    print "#{' '*label_width + caption}".blue unless caption.empty?
    report = ColoredReport.new(label_width, format)
    results = yield(report)
    print "#{'-'*label_width}\n".yellow
    Array === results and results.grep(Tms).each { |t|
      print((labels.shift || t.label || "").ljust(label_width).yellow, t.format(format.yellow))
    }
    report.list
  end


  def self.pbm(label_width = 0, *labels, &blk)
    penchmark(CAPTION, label_width, FORMAT, *labels, &blk)
  end
end